"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const type_graphql_1 = require("type-graphql");
const constants_1 = require("./constants");
const request_constants_1 = require("@nestjs/core/router/request/request-constants");
let TypeGraphQLOptionsFactory = class TypeGraphQLOptionsFactory {
    constructor(moduleRef, modulesContainer, rootModuleOptions) {
        this.moduleRef = moduleRef;
        this.modulesContainer = modulesContainer;
        this.rootModuleOptions = rootModuleOptions;
    }
    createGqlOptions() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const globalResolvers = type_graphql_1.getMetadataStorage().resolverClasses.map(metadata => metadata.target);
            const featureModuleOptionsArray = [];
            const resolvers = [];
            const providersMetadataMap = new Map();
            for (const module of this.modulesContainer.values()) {
                for (const provider of module.providers.values()) {
                    if (typeof provider.name === "string" &&
                        provider.name.includes(constants_1.TYPEGRAPHQL_FEATURE_MODULE_OPTIONS)) {
                        featureModuleOptionsArray.push(provider.instance);
                    }
                    if (globalResolvers.includes(provider.metatype)) {
                        providersMetadataMap.set(provider.metatype, provider);
                        resolvers.push(provider.metatype);
                    }
                }
            }
            const orphanedTypes = common_1.flatten(featureModuleOptionsArray.map(it => it.orphanedTypes));
            const container = {
                get: (cls, { context }) => {
                    let contextId = context[request_constants_1.REQUEST_CONTEXT_ID];
                    if (!contextId) {
                        contextId = core_1.ContextIdFactory.create();
                        context[request_constants_1.REQUEST_CONTEXT_ID] = contextId;
                    }
                    const providerMetadata = providersMetadataMap.get(cls);
                    if (providerMetadata.isDependencyTreeStatic() &&
                        !providerMetadata.isTransient) {
                        return this.moduleRef.get(cls, { strict: false });
                    }
                    return this.moduleRef.resolve(cls, contextId, { strict: false });
                },
            };
            const schema = yield type_graphql_1.buildSchema(Object.assign(Object.assign({}, this.rootModuleOptions), { resolvers: resolvers, orphanedTypes,
                container }));
            return Object.assign(Object.assign({}, this.rootModuleOptions), { schema });
        });
    }
};
TypeGraphQLOptionsFactory = tslib_1.__decorate([
    common_1.Injectable(),
    tslib_1.__param(2, common_1.Inject(constants_1.TYPEGRAPHQL_ROOT_MODULE_OPTIONS)),
    tslib_1.__metadata("design:paramtypes", [core_1.ModuleRef,
        core_1.ModulesContainer, Object])
], TypeGraphQLOptionsFactory);
exports.default = TypeGraphQLOptionsFactory;
